import argparse
from ctypes import CDLL, c_ulong
import random, time

address = { 'CDC': 0x17 , 'ARICH': 0x12 } 

lli = CDLL("libpcie40.so")
slc_read = lli.pcie40_readfee32
slc_read.restype = c_ulong

parser = argparse.ArgumentParser( description = 'Test slow control: read/write random values in register 0x17' )
parser.add_argument('N',type=int,help='Number of events')
parser.add_argument('--channel', '-c' , help = 'Channel number' , type = int , default = 0 )
parser.add_argument('--detector' , '-d' , help = 'Detector' , default = 'CDC' )
parser.add_argument('--readonly' , '-r' , help = 'Read only (default read/write)' , action='store_const', const='ro', default='rw' )

args = parser.parse_args()
result = 0 
lli.ecs_open( 0 , 2 )

random.seed()

nerr = 0 

for i in range( args.N ):
    data = random.randint( 0 , 0xFFFFFFFF )
    if args.readonly == 'rw': 
        lli.pcie40_writefee32( 0 , args.channel , address[ args.detector ] , data )
    result = slc_read( 0 , args.channel , address[ args.detector ] )
    if i%10000 == 0:
        print(str(i))
    if args.readonly == 'rw': 
        if result != data:
            print("mismatch between write ({0:X}) and read ({1:X})".format( data , result ))
            nerr = nerr + 1

lli.ecs_close( 0 , 2 )

print ("Total number of errors = " + str(nerr))
